#include "brimstone/graphics/pipeline.h"
#include <pch.h>

int main(int argc, char** argv) {
    InitSystems();

    SDL_GPUDevice* device = DeviceCreate();
    assert(device != NULL);

    SDL_Window* window = SDL_CreateWindow("Brimstone", 800, 600, 0);
    if(window == NULL) {
        SDL_Log("SDL_CreateWindow: %s", SDL_GetError());
        exit(EXIT_FAILURE);
    }
    SDL_ClaimWindowForGPUDevice(device, window);

    /*Vertex vertices[] = {
        { -0.5f, -0.5f, 0.0f, 1.0f, 0.0f, 0.0f },
        { 0.5f, -0.5f, 0.0f, 0.0f, 1.0f, 0.0f },
        { 0.0f, 0.5f, 0.0f, 0.0f, 0.0f, 1.0f },
    };*/
    float vertices[3][3] = {
        { -0.5f, -0.5f, 0.0f },
        { 0.5f, -0.5f, 0.0f },
        { 0.0f, 0.5f, 0.0f },
    };

    float colors[3][3] = {
        { 1.0f, 0.0f, 0.0f },
        { 0.0f, 1.0f, 0.0f },
        { 0.0f, 0.0f, 1.0f },
    };

    SDL_GPUBuffer* vertex_buffer = SDL_CreateGPUBuffer(device,
        &(SDL_GPUBufferCreateInfo){
            .usage = SDL_GPU_BUFFERUSAGE_VERTEX,
            .size = sizeof(vertices),
        });

    bool upload_successful = UploadToBuffer(device, vertices, sizeof(vertices), vertex_buffer);
    assert(upload_successful);

    SDL_GPUBuffer* color_buffer = SDL_CreateGPUBuffer(device,
        &(SDL_GPUBufferCreateInfo){
            .usage = SDL_GPU_BUFFERUSAGE_VERTEX,
            .size = sizeof(colors),
        });

    upload_successful = UploadToBuffer(device, colors, sizeof(colors), color_buffer);
    assert(upload_successful);

    SDL_GPUShader* vertex_shader = ShaderLoad(device, "shader.vert.spv", SDL_GPU_SHADERSTAGE_VERTEX);
    assert(vertex_shader != NULL);

    SDL_GPUShader* fragment_shader = ShaderLoad(device, "shader.frag.spv", SDL_GPU_SHADERSTAGE_FRAGMENT);
    assert(fragment_shader != NULL);

    SDL_GPUGraphicsPipeline* pipeline = GraphicsPipelineCreate(
        device, window, vertex_shader, fragment_shader);
    assert(pipeline != NULL);

    bool running = true;
    SDL_Event event;
    while(running) {
        while(SDL_PollEvent(&event)) {
            switch(event.type) {
            case SDL_EVENT_QUIT:
                running = false;
                break;
            }
        }

        SDL_GPUCommandBuffer* cmd = SDL_AcquireGPUCommandBuffer(device);
        if(cmd == NULL) {
            SDL_Log("SDL_AcquireGPUCommandBuffer: %s", SDL_GetError());
            exit(EXIT_FAILURE);
        }

        SDL_GPUTexture* swapchain;
        uint32_t swapchain_width, swapchain_height;
        SDL_AcquireGPUSwapchainTexture(cmd, window, &swapchain, &swapchain_width, &swapchain_height);

        SDL_GPURenderPass* render_pass = SDL_BeginGPURenderPass(cmd,
            &(SDL_GPUColorTargetInfo){
                .texture = swapchain,
                .load_op = SDL_GPU_LOADOP_CLEAR,
                .clear_color = (SDL_FColor){ .r = 0.1f, .g = 0.3f, .b = 0.4f, .a = 1.0f },
            },
            1, NULL);
        if(render_pass == NULL) {
            SDL_Log("SDL_BeginGPURenderPass: %s", SDL_GetError());
            exit(EXIT_FAILURE);
        }

        SDL_BindGPUGraphicsPipeline(render_pass, pipeline);
        SDL_BindGPUVertexBuffers(render_pass, 0,
            (SDL_GPUBufferBinding[]){
                {
                    .buffer = vertex_buffer,
                    .offset = 0,
                },
                {
                    .buffer = color_buffer,
                    .offset = 0,
                },
            },
            2);
        SDL_DrawGPUPrimitives(render_pass, 3, 1, 0, 0);

        SDL_EndGPURenderPass(render_pass);

        if(!SDL_SubmitGPUCommandBuffer(cmd)) {
            SDL_Log("SDL_SubmitGPUCommandBuffer: %s", SDL_GetError());
            exit(EXIT_FAILURE);
        }
    }

    return 0;
}
